-------------------------------------------------------------------------------
-- Concurrent Haskell Stepper
--   StepMain Module
--   
-------------------------------------------------------------------------------

module CHS.StepMain
  (
    executeIO
  )
  where


-------------------------------------------------------------------------------
-- IMPORTS
-------------------------------------------------------------------------------

import Prelude hiding ( IO, print, putStr, getChar, getLine )
import qualified Prelude as P
import qualified Control.Concurrent as C
import qualified ConcurrentDebug as CD

import CHS.BaseTypes
import CHS.ComMVars


-------------------------------------------------------------------------------
-- TYPES
-------------------------------------------------------------------------------


------------------------------------------------------------------------------
-- FUNCTIONS
-------------------------------------------------------------------------------

executeIO :: IO () -> P.IO ()
executeIO tree = do
  myId <- C.myThreadId
  executeThread (myId,tree)


executeThread :: Thread -> P.IO ()
executeThread thread = do
  result <- stepThread thread
  case result of
       NotStepped thread' -> executeThread thread'
       Stepped thread'	  -> C.putMVar treeMVar result >>
			     executeThread thread'
       Fork _ thread'	  -> C.putMVar treeMVar result >>
			     executeThread thread'
       Kill _ thread'	  -> C.putMVar treeMVar result >>
			     executeThread thread'
       _		  -> error "leerer Thread"


stepThread :: Thread -> P.IO (Result Thread)
stepThread (_,Return _) =
  return Terminate

stepThread (tId,ConcAction _ dA _ _) = do
  mvar <- dA
  return $ Stepped (tId,Return mvar)
 
stepThread (tId,SeqAction _ sA) = do
  v <- sA
  return $ NotStepped (tId,Return v)

stepThread (tId,ForkAction tree fA _) = do
  (v,newId) <- fA
  return $ Fork (newId,tree) (tId,Return v)
 
stepThread (tId,KillAction threadId kA _) = do
  v <- kA 
  return $ Kill threadId (tId,Return v)

stepThread (tId,Bind (ForkAction tree forkA _) fa) = do
  (v,newId) <- forkA
  return $ Fork (newId,tree) (tId,fa v)

stepThread (tId,Bind (Return a) fa) =
  return $ NotStepped (tId,fa a)

stepThread (tId,Bind (ConcAction _ dA _ _) fa) = do
  mvar <- dA
  return $ Stepped (tId,fa mvar)

stepThread (tId,Bind (SeqAction _ sA) fa) = do
  v <- sA
  return $ NotStepped (tId,fa v)

stepThread (tId,Bind (KillAction threadId kA _) fa) = do
  v <- kA
  return $ Kill threadId (tId,fa v)

stepThread (tId,Bind (Bind fa1 fa2) fa3) = do
  stepThread (tId,Bind fa1 (\x->(Bind (fa2 x) fa3)))
  
