-------------------------------------------------------------------------------
-- Concurrent Haskell Stepper
--   StepperSampleVar Module
--   
-------------------------------------------------------------------------------

module CHS.StepperSampleVar
  (
   SampleVar(..),
   newEmptySampleVar,		-- :: IO (SampleVar a)
   newEmptySampleVarLine,	-- :: CodePosition -> IO (SampleVar a)
   newSampleVar,		-- :: a -> IO (SampleVar a)
   newSampleVarLine,		-- :: CodePosition -> a -> IO (SampleVar a)
   emptySampleVar,		-- :: SampleVar a -> IO ()
   emptySampleVarLine,		-- :: CodePosition -> SampleVar a -> IO ()
   writeSampleVar,		-- :: SampleVar a -> a -> IO ()
   writeSampleVarLine,		-- :: CodePosition -> SampleVar a -> a -> IO ()
   readSampleVar,		-- :: SampleVar a -> IO a
   readSampleVarLine,		-- :: CodePosition -> SampleVar a -> IO a
   isEmptySampleVar		-- :: C.SampleVar a -> P.IO Bool
  )
  where


-------------------------------------------------------------------------------
-- IMPORTS
-------------------------------------------------------------------------------

import Prelude hiding ( IO, print, putStr, putStrLn, getChar, getLine )
import qualified Prelude as P
import qualified Control.Concurrent as C 
import qualified ConcurrentDebug as CD
import qualified System.IO as SI

import CHS.ComMVars
import CHS.BaseTypes

import System.IO.Unsafe


-------------------------------------------------------------------------------
-- TYPES
-------------------------------------------------------------------------------

data SampleVar a = SampleVar (CD.SampleVar a) (C.SampleVar a) (C.SampleVar ())


-------------------------------------------------------------------------------
-- FUNCTIONS
-------------------------------------------------------------------------------

newSampleVarNoMVar :: C.MVar SampleVarNo
newSampleVarNoMVar = unsafePerformIO $ C.newMVar (SampleVarNo (-1))


newEmptySampleVar :: IO (SampleVar a)
newEmptySampleVar = newEmptySampleVarLine NoPosition


newEmptySampleVarLine :: CodePosition -> IO (SampleVar a)
newEmptySampleVarLine pos = 
  ConcAction (SampleVarObj (SampleVarNo (-1)))
	     (do cdSampleVar <- CD.newEmptySampleVarLine pos
	         C.putMVar breakMVar ()
		 C.takeMVar finishedBreakMVar
		 searchSampleVar <- C.newEmptySampleVar
		 readStop <- C.newEmptySampleVar
		 return $ SampleVar cdSampleVar searchSampleVar readStop)
	     (do cSampleVar <- C.newEmptySampleVar
		 searchSampleVar2 <- C.newEmptySampleVar
		 readStop2 <- C.newEmptySampleVar
		 samplevarNo <- C.takeMVar newSampleVarNoMVar
		 let cdSampleVar2 = CD.SampleVar (SampleVarNo (-1)) cSampleVar
		     samplevar2 = SampleVar cdSampleVar2 searchSampleVar2 
					    readStop2
		 C.putMVar newSampleVarNoMVar (pred samplevarNo)	    
		 return $ Just (samplevar2,samplevar2))
             (\_ -> do samplevarNo <- C.takeMVar newSampleVarNoMVar
		       C.putMVar newSampleVarNoMVar (succ samplevarNo))


newSampleVar :: a -> IO (SampleVar a)
newSampleVar x = newSampleVarLine NoPosition x


newSampleVarLine :: CodePosition -> a -> IO (SampleVar a)
newSampleVarLine pos x = 
  ConcAction (SampleVarObj (SampleVarNo (-1)))
	     (do cdSampleVar <- CD.newSampleVarLine pos x
	         C.putMVar breakMVar ()
		 C.takeMVar finishedBreakMVar
		 searchSampleVar <- C.newSampleVar x
		 readStop <- C.newSampleVar ()
		 return $ SampleVar cdSampleVar searchSampleVar readStop)
	     (do cSampleVar <- C.newSampleVar x
		 searchSampleVar2 <- C.newSampleVar x
		 readStop2 <- C.newSampleVar ()
		 let cdSampleVar2 = CD.SampleVar (SampleVarNo (-1)) cSampleVar
		     samplevar2 = SampleVar cdSampleVar2 searchSampleVar2 
					    readStop2
		 return $ Just (samplevar2,samplevar2))
             (\_ -> do samplevarNo <- C.takeMVar newSampleVarNoMVar
		       C.putMVar newSampleVarNoMVar (succ samplevarNo))


emptySampleVar :: SampleVar a -> IO ()
emptySampleVar = emptySampleVarLine NoPosition


emptySampleVarLine :: CodePosition -> SampleVar a -> IO ()
emptySampleVarLine pos s@(SampleVar cdSampleVar@(CD.SampleVar samplevarNo _) 
		   searchSampleVar readStop) =
  ConcAction (SampleVarObj samplevarNo)
	     (do CD.emptySampleVarLine pos cdSampleVar
		 C.putMVar breakMVar ()
		 myEmptySampleVar readStop
		 C.takeMVar finishedBreakMVar
		 myEmptySampleVar searchSampleVar)
	     (do b <- isEmptySampleVar searchSampleVar
                 if b
                    then return $ Just ((),Nothing)
                    else do v <- C.readSampleVar searchSampleVar
		    	    --C.emptySampleVar searchSampleVar
			    return $ Just ((),Just v))
             (maybe (return ()) (C.writeSampleVar searchSampleVar))


writeSampleVar :: SampleVar a -> a -> IO ()
writeSampleVar = writeSampleVarLine NoPosition


writeSampleVarLine :: CodePosition -> SampleVar a -> a -> IO ()
writeSampleVarLine pos (SampleVar cdSampleVar@(CD.SampleVar samplevarNo _) 
		        searchSampleVar readStop) value = 
  ConcAction (SampleVarObj samplevarNo)
	     (do CD.writeSampleVarLine pos cdSampleVar value
		 C.putMVar breakMVar ()
		 C.writeSampleVar readStop ()
		 C.takeMVar finishedBreakMVar
		 C.writeSampleVar searchSampleVar value)
	     (do --P.putStrLn "write Anfang"
		 b <- isEmptySampleVar searchSampleVar
		 if b
		    then do C.writeSampleVar searchSampleVar value
			    --P.putStrLn "write Ende"
			    return $ Just ((),Nothing)
		    else do v <- C.readSampleVar searchSampleVar
			    C.writeSampleVar searchSampleVar value
			    --P.putStrLn "write Ende"
			    return $ Just ((),Just v))
             (maybe (myEmptySampleVar searchSampleVar)
		    (C.writeSampleVar searchSampleVar))


readSampleVar :: SampleVar a -> IO a
readSampleVar = readSampleVarLine NoPosition


readSampleVarLine :: CodePosition -> SampleVar a -> IO a
readSampleVarLine pos (SampleVar cdSampleVar@(CD.SampleVar samplevarNo _) 
		       searchSampleVar readStop) =
  ConcAction (SampleVarObj samplevarNo)
	     (do CD.readSampleVarLine pos cdSampleVar
		 C.readSampleVar readStop
		 C.putMVar breakMVar ()
		 C.takeMVar finishedBreakMVar
		 C.readSampleVar searchSampleVar)
	     (do --P.putStrLn "read Anfang"
		 b <- isEmptySampleVar searchSampleVar
                 if b
                    then return Nothing
                    else do v <- C.readSampleVar searchSampleVar
			    return $ Just (v,v))
             (\v -> do b <- isEmptySampleVar searchSampleVar
		       b2 <- C.isEmptyMVar searchSampleVar
		       P.putStrLn $ "emptyMVar? " ++ show b2
		       (readers,val) <- C.takeMVar searchSampleVar
		       case readers of
			    1 -> P.putStrLn "1" >>
				 C.swapMVar val v >>
				 P.putStrLn "2" >>
			         C.putMVar searchSampleVar (1,val)
			    _ -> P.putStrLn "3" >>
				 C.putMVar val v >>
				 P.putStrLn "4" >>
			         C.putMVar searchSampleVar (min 1 (readers+1), val) 
		       P.putStrLn $ "vor write " ++ show b
		       P.putStrLn "nach write")


isEmptySampleVar :: C.SampleVar a -> P.IO Bool
isEmptySampleVar sampleVar = do
  (value,x) <- C.readMVar sampleVar
  return (value <= 0)


myEmptySampleVar :: C.SampleVar a -> P.IO ()
myEmptySampleVar v = do
   (readers, var) <- C.takeMVar v
   if readers > 0 then
     C.takeMVar var >>
     C.putMVar v (0,var)
    else
     C.putMVar v (readers,var)