module QSem
	( -- * Simple Quantity Semaphores
	  QSem,		       -- abstract
	  newQSem,	       -- :: Int  -> IO QSem
	  waitQSem,	       -- :: QSem -> IO ()
	  signalQSem,	       -- :: QSem -> IO ()
	  getQuantityQSem      -- :: QSem -> IO Int
	) where


import Prelude
import Control.Concurrent.MVar


newtype QSem = QSem (MVar (Int, [MVar ()]))


-- |Build a new 'QSem'
newQSem :: Int -> IO QSem
newQSem init = do
   sem <- newMVar (init,[])
   return (QSem sem)


-- |Wait for a unit to become available
waitQSem :: QSem -> IO ()
waitQSem (QSem sem) = do
   (avail,blocked) <- takeMVar sem  -- gain ex. access
   if avail > 0 then
     putMVar sem (avail-1,[])
    else do
     block <- newEmptyMVar
      {-
	Stuff the reader at the back of the queue,
	so as to preserve waiting order. A signalling
	process then only have to pick the MVar at the
	front of the blocked list.

	The version of waitQSem given in the paper could
	lead to starvation.
      -}
     putMVar sem (0, blocked++[block])
     takeMVar block


-- |Signal that a unit of the 'QSem' is available
signalQSem :: QSem -> IO ()
signalQSem (QSem sem) = do
   (avail,blocked) <- takeMVar sem
   case blocked of
     [] -> putMVar sem (avail+1,[])

     (block:blocked') -> do
	   putMVar sem (0,blocked')
	   putMVar block ()


getQuantityQSem :: QSem -> IO Int
getQuantityQSem (QSem sem) = do
  (avail,_) <- readMVar sem
  return avail