module Tree
      (
	Tree(..),
	Edge(..),
	Attribute(..),
	Style(..),
	Color(..),
	addSuccessor,
	toDottyFile,
	toDotty
      )
where


data Tree = Node [Attribute] [Edge]


data Edge = Edge [Attribute] Tree
	      

data Attribute = Label String
	       | Style Style
	       | Color Color

instance Show Attribute where
  show (Label l) = "label=\"" ++ l ++ "\""
  show (Style s) = "style=" ++ show s
  show (Color c) = "color=" ++ show c


data Style = Bold | Dotted | Dashed | Filled
  
instance Show Style where
  show Bold = "bold"
  show Dotted = "dotted"
  show Filled = "filled"
  show Dashed = "dashed"


data Color = Red | Blue | Green | Yellow | Black
  
instance Show Color where
  show Red = "red"
  show Blue = "blue"
  show Green = "green" 
  show Yellow = "yellow"
  show Black = "black"


addSuccessor :: Tree -> [Attribute] -> [Attribute] -> Tree -> Tree
addSuccessor (Node attributes edges) newAttr attributes' tree =
  (Node (attributes++newAttr) (Edge attributes' tree:edges)) 


toDottyFile :: Tree -> String
toDottyFile tree =
  "digraph G {\n" ++ fst (toDotty 0 1 tree) ++ "\n}\n"


toDotty :: Int -> Int -> Tree -> (String,Int)
toDotty no1 no2 (Node attributes (Edge attributes' node2:edges)) =
  let edgeLabel = show no1 ++ " -> " ++ show no2 ++ show attributes' ++ ";\n"
      (string,no) = toDotty no1 (no2+1) (Node attributes edges)
      (string',no') = toDotty no2 no node2 in
  (edgeLabel ++ string ++ string',no')

toDotty no1 no2 (Node attributes []) = 
  (show no1 ++ " " ++ show attributes ++ ";\n",no2)

toDotty _ _ _=
  error ("ERROR")
  

